%%%-------------------------------------------------------------------
%%% @author Piotr Duleba
%%% @copyright (C) 2021 ACK CYFRONET AGH
%%% This software is released under the MIT license
%%% cited in 'LICENSE.txt'.
%%% @end
%%%-------------------------------------------------------------------
%%% @doc
%%% This file provides functions for rpc test modules.
%%% @end
%%%-------------------------------------------------------------------
-module(test_rpc).
-author("Piotr Duleba").

-export([call/4]).


%%%===================================================================
%%% API functions
%%%===================================================================


-spec call(op_worker | oz_worker | onepanel, oct_background:node_selector(), atom(), [term()]) -> term().
call(Service, NodeSelector, Function, Args) ->
    Node = case is_known_node(NodeSelector) of
        true ->
            NodeSelector;
        false ->
            lists_utils:random_element(get_service_nodes(Service, NodeSelector))
    end,

    case rpc:call(Node, test_rpc_api, Function, Args) of
        {badrpc, Error} -> error({test_rpc_failed, Node, Error});
        __Result -> __Result
    end.

%%%===================================================================
%%% Helpers
%%%===================================================================


%% @private
-spec is_known_node(oct_background:node_selector()) -> boolean().
is_known_node(NodeSelector) ->
    lists:member(NodeSelector, nodes(known)).


%% @private
-spec get_service_nodes(op_worker | oz_worker | onepanel, oct_background:node_selector()) -> [node()].
get_service_nodes(Service, NodeSelector) ->
    case Service of
        op_worker -> oct_background:get_provider_nodes(NodeSelector);
        oz_worker -> oct_background:get_zone_nodes();
        onepanel -> oct_background:get_service_panels(NodeSelector)
    end.
